﻿local addonName, addon = ...

do
	local exceptions = {
		RUNE_REGEN = {
			DEATHKNIGHT = true
		},
		FOCUS_REGEN = {
			HUNTER = true
		},
		ALTERNATEMANA = {
			DRUID = true,
			MONK = true,
		},
		ENERGY_REGEN = {
			ROGUE = true,
			DRUID = true,
			MONK = true,
		},
	}
	
	local _, class = UnitClass("player")
	for k, v in pairs(PAPERDOLL_STATCATEGORIES) do
		local stats = v.stats
		for i = #stats, 1, -1 do
			local v = exceptions[stats[i]]
			if v and not v[class] then
				tremove(stats, i)
			end
		end
	end
end

local DEFAULT_CATEGORY_ORDER = CopyTable(PAPERDOLL_STATCATEGORY_DEFAULTORDER)
local DEFAULT_CATEGORIES = CopyTable(PAPERDOLL_STATCATEGORIES)

local db
local statsDB = addon.statsDB
local cvarOrder, cvarCollapsed = "statCategoryOrder", "statCategoriesCollapsed"

local FRAME_HEIGHT_BASE = 78
local DROPDOWN_OFFSET = 28

addon = CreateFrame("Frame", "PDFStats", UIParent)
addon:SetToplevel(true)
addon:EnableMouse(true)
addon:SetMovable(true)
addon:RegisterForDrag("LeftButton")
addon:SetSize(256, FRAME_HEIGHT_BASE + DROPDOWN_OFFSET)
addon:SetPoint("CENTER")
addon:Hide()
addon:SetScript("OnDragStart", addon.StartMoving)
addon:SetScript("OnDragStop", addon.StopMovingOrSizing)
addon:SetScript("OnHide", addon.StopMovingOrSizing)
addon:SetBackdrop({
	bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	edgeSize = 14,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
})
addon:SetBackdropColor(0, 0, 0, 0.8)
addon:SetBackdropBorderColor(0.3, 0.3, 0.3, 1)
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

-- prevent default UI from messing stuff up while dual profile is disabled
PaperDollFrame:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")


local frameList = CreateFrame("Frame", "PDFStatsFrameList", addon, "UIDropDownMenuTemplate")
frameList:SetPoint("TOPLEFT", -8, -8)
frameList.statLists = {}

local closeButton = CreateFrame("Button", nil, addon, "UIPanelCloseButton")
closeButton:SetPoint("TOPRIGHT", -2, -4)

local deleteButton = CreateFrame("Button", nil, addon, "UIPanelButtonTemplate")
deleteButton:SetSize(80, 22)
deleteButton:SetPoint("RIGHT", closeButton, "LEFT", 0, -1)
deleteButton:SetText(DELETE)
deleteButton:Disable()
deleteButton:SetScript("OnClick", function(self)
	addon:DeleteCategory(addon:GetSelectedCategory())
end)

local resetButton = CreateFrame("Button", nil, addon, "UIPanelButtonTemplate")
resetButton:SetSize(80, 22)
resetButton:SetPoint("TOP", deleteButton, "BOTTOM", 0, -4)
resetButton:SetText(RESET)
resetButton:Disable()
resetButton:SetScript("OnClick", function(self)
	local selectedCategory = addon:GetSelectedCategory()
	addon:ResetCategory(selectedCategory)
	addon:UpdateGUI()
end)

local dualProfile = CreateFrame("CheckButton", nil, addon, "OptionsBaseCheckButtonTemplate")
dualProfile:SetPoint("TOPLEFT", frameList, "BOTTOMLEFT", 16, 0)
dualProfile:SetPushedTextOffset(0, 0)
dualProfile:SetDisabledFontObject("GameFontDisable")
dualProfile.onClick = function(self)
	local checked = not not self:GetChecked()
	addon.db.dualProfile = checked
	local profile = addon.db.profile
	if checked then
		local activeSpec = GetActiveSpecGroup()
		if not profile[2] then
			-- copy the primary spec profile to current spec profile and copy defaults to inactive spec profile
			profile[activeSpec] = profile[1]
			profile[3 - activeSpec] = addon:CopyProfileDefaults()
		end
	else
		-- remove secondary spec profile and use current spec profile for both specs
		if profile[2] then
			profile[1] = profile[GetActiveSpecGroup()]
			profile[2] = nil
		end
		db = profile[1]
	end
	addon:CheckTalentSpec()
end
dualProfile:SetScript("OnClick", dualProfile.onClick)
dualProfile.tooltipText = "If disabled, will use the same layout for both primary and secondary talents."

local text = dualProfile:CreateFontString(nil, nil, "GameFontHighlight")
text:SetPoint("LEFT", dualProfile, "RIGHT", 0, 1)
dualProfile:SetFontString(text)
dualProfile:SetText("Use dual profile")

local info = addon:CreateFontString(nil, nil, "GameFontNormalSmall")
info:SetPoint("TOP", dualProfile, "BOTTOM")
info:SetPoint("BOTTOMLEFT", 8, 8)
info:SetPoint("BOTTOMRIGHT", -8, 8)
info:SetJustifyH("LEFT")
info:SetJustifyV("TOP")
info:SetText("This stat group has been removed, and cannot be modified until restored. Click Reset to restore it.")
info:Hide()

local editBox = CreateFrame("EditBox", "PDFStatsEditBox", addon, "InputBoxTemplate")
editBox:SetAutoFocus(true)
editBox:SetFontObject(ChatFontSmall)
editBox:SetSize(160, 20)
editBox:SetPoint("TOPLEFT", dualProfile, "BOTTOMLEFT", 8, -4)
editBox:SetText("Enter category name...")
editBox:Hide()
editBox:SetScript("OnEnterPressed", function(self)
	local text = self:GetText()
	if text:match("%S+") then
		if not addon:CreateCategory(text) then
			self:SetText("Enter category name...")
			self:HighlightText()
			return
		end
		addon:SelectCategory(text)
		UIDropDownMenu_SetText(frameList, text)
		self:Hide()
	else
		self:SetText("Enter category name...")
		self:HighlightText()
		addon:Print("Invalid category name.")
	end
end)
editBox:SetScript("OnEscapePressed", function(self)
	self:Hide()
	addon:SelectCategory(nil)
end)

-- onClick function for all dropdown menu items
local function onClick(self)
	local id = self.owner.id
	local stats = addon:GetSelectedStats()
	if self.value ~= "none" then
		stats[id] = self.value
	else
		tremove(stats, id)
	end
	addon:UpdateGUI()
	CloseDropDownMenus()
	PaperDollFrame_UpdateStats()
end

-- dropdown menu initialize function that adds all the menu items
local function initialize(self, level)
	if level == 1 then
		-- menu level 1; categories
		local info = UIDropDownMenu_CreateInfo()
		info.text = NONE
		info.value = "none"
		info.func = onClick
		info.owner = self
		UIDropDownMenu_AddButton(info, level)
		
		for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = _G["STAT_CATEGORY_"..v]
			info.value = v
			info.notCheckable = true
			-- info.notClickable = true
			info.hasArrow = true
			UIDropDownMenu_AddButton(info, level)
		end
	elseif level == 2 then
		-- menu level 2; stats
		local info = UIDropDownMenu_CreateInfo()
		info.text = _G["STAT_CATEGORY_"..UIDROPDOWNMENU_MENU_VALUE]
		info.isTitle = true
		info.notCheckable = true
		UIDropDownMenu_AddButton(info, level)
		
		local category = statsDB[UIDROPDOWNMENU_MENU_VALUE]
		for i, stat in ipairs(category) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = stat.label
			info.value = stat.value
			info.func = onClick
			info.owner = self
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

local function onEnter(self) self:SetAlpha(0.6) end
local function onLeave(self) self:SetAlpha(0.4) end

local function createArrow(parent)
	local btn = CreateFrame("Button", nil, parent)
	btn:SetSize(16, 8)
	btn:SetAlpha(0.4)
	btn:SetScript("OnEnter", onEnter)
	btn:SetScript("OnLeave", onLeave)
	local tx = btn:CreateTexture(nil, "BACKGROUND")
	tx:SetTexture([[Interface\PaperDollInfoFrame\StatSortArrows]])
	tx:SetAllPoints()
	btn.tx = tx
	btn:SetNormalTexture(tx)
	return btn
end

local function arrowOnClick(self)
	local stats = addon:GetSelectedStats()
	local id = self:GetParent().id
	local swap = id + self.modifier
	stats[id], stats[swap] = stats[swap], stats[id]
	addon:UpdateGUI()
	PaperDollFrame_UpdateStats()
end

function addon:NewStat(index)
	local frame = CreateFrame("Frame", "PDFStatsFrameListStat"..index, self, "UIDropDownMenuTemplate")
	if index == 1 then
		frame:SetPoint("LEFT", frameList)
		frame:SetPoint("TOP", dualProfile, "BOTTOM", 0, -4)
	else
		frame:SetPoint("TOP", frameList.statLists[index - 1], "BOTTOM", 0, 4)
	end
	frame.id = index
	frame:SetFrameLevel(self:GetFrameLevel() + 1)
	frame:Hide()
	
	UIDropDownMenu_Initialize(frame, initialize)
	
	UIDropDownMenu_SetWidth(frame, 192)
	UIDropDownMenu_JustifyText(frame, "LEFT")
	UIDropDownMenu_SetSelectedValue(frame, "none")

	local up = createArrow(frame)
	up:SetPoint("BOTTOMLEFT", frame, "RIGHT", -8, 2)
	up.tx:SetTexCoord(0, 1, 0, 0.5)
	up:SetScript("OnClick", arrowOnClick)
	up.modifier = -1
	frame.up = up

	local down = createArrow(frame)
	down:SetPoint("TOP", up, "BOTTOM")
	down.tx:SetTexCoord(0, 1, 0.5, 1)
	down:SetScript("OnClick", arrowOnClick)
	down.modifier = 1
	frame.down = down

	frameList.statLists[index] = frame
	return frame
end


local function newCategory()
	addon:SelectCategory(nil)
	editBox:SetText("Enter category name...")
	editBox:Show()
end

local function onClick(self)
	addon:SelectCategory(self.value)
	editBox:Hide()
end

function frameList:initialize()
	-- add default stat frames
	for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
		local info = UIDropDownMenu_CreateInfo()
		info.text = _G["STAT_CATEGORY_"..v]
		info.value = v
		info.func = onClick
		info.colorCode = addon:IsHiddenCategory(v) and GRAY_FONT_COLOR_CODE
		UIDropDownMenu_AddButton(info)
	end

	for i, v in ipairs(db.registry) do
		if not DEFAULT_CATEGORIES[v] then
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.func = onClick
			info.colorCode = "|cff56a3ff"
			UIDropDownMenu_AddButton(info)
		end
	end
	
	local info = UIDropDownMenu_CreateInfo()
	info.text = "Create new..."
	info.value = "new"
	info.func = newCategory
	info.notCheckable = true
	UIDropDownMenu_AddButton(info)
end

UIDropDownMenu_JustifyText(frameList, "LEFT")
UIDropDownMenu_SetText(frameList, "Select category")


hooksecurefunc("SetCVar", function(cvar, value)
	if cvar == cvarOrder then
		db.statCategoryOrder = value
	end
end)

hooksecurefunc("SetCVarBitfield", function(cvar, bit, value)
	if cvar == cvarCollapsed then
		if bit > #PAPERDOLL_STATCATEGORY_DEFAULTORDER then
			db.collapsed[bit] = nil
		else
			db.collapsed[bit] = value
		end
	end
end)


SlashCmdList.PDFSTATS_TOGGLE = function()
	addon:SetShown(not addon:IsShown())
end
SLASH_PDFSTATS_TOGGLE1 = "/pdf"
SLASH_PDFSTATS_TOGGLE2 = "/stats"


local profileDefaults = {
	registry = DEFAULT_CATEGORY_ORDER,
	categories = {},
	statCategoryOrder = "",
	collapsed = {},
}

for i, v in ipairs(DEFAULT_CATEGORY_ORDER) do
	profileDefaults.categories[v] = DEFAULT_CATEGORIES[v].stats
end

local defaults = {
	profile = {profileDefaults},
	dualProfile = true,
}

function addon:ADDON_LOADED(addon)
	if addon == addonName then
		PDFStatsDB = PDFStatsDB or CopyTable(defaults)

		db = PDFStatsDB
		db.hidden = nil
		
		if not db.profile then
			PDFStatsDB = {dualProfile = true, profile = {db}}
		end
		
		self.db = PDFStatsDB
		
		-- clear out any deprecated stats
		for i, profile in ipairs(self.db.profile) do
			for k, category in pairs(profile.categories) do
				for i = #category, 1, -1 do
					if not PAPERDOLL_STATINFO[category[i]] then
						tremove(category, i)
					end
				end
			end
		end
		
		self:UnregisterEvent("ADDON_LOADED")
	end
end

-- copies profile defaults to the currently inactive profile
function addon:CopyProfileDefaults()
	return CopyTable(profileDefaults)
end

function addon:LoadSettings()
	for i, name in ipairs(DEFAULT_CATEGORY_ORDER) do
		if not db.categories[name] then
			self:FixCategoryIndex(name)
		end
	end
	
	-- insert our stored custom categories into table of default categories
	for i, v in ipairs(db.registry) do
		if DEFAULT_CATEGORIES[v] then
			PAPERDOLL_STATCATEGORIES[v].id = i
			PAPERDOLL_STATCATEGORIES[v].stats = db.categories[v]
		else
			self:InjectCategoryData(v, db.categories[v])
		end
	end
	
	if db.statCategoryOrder == "" then
		local order = {}
		for i = 1, #PAPERDOLL_STATCATEGORY_DEFAULTORDER do
			order[i] = i
		end
		db.statCategoryOrder = table.concat(order, ",")
	end
	
	SetCVar(cvarOrder, db.statCategoryOrder)
	
	for i, v in ipairs(PAPERDOLL_STATCATEGORY_DEFAULTORDER) do
		SetCVarBitfield(cvarCollapsed, i, db.collapsed[i] or false)
	end
	
	self:InitStatCategories()
end

function addon:PLAYER_LOGIN()
	-- inserting this on PLAYER_LOGIN since UnitPowerType doesn't return anything earlier
	for i, v in ipairs(statsDB["GENERAL"]) do
		if v.value == "POWER" then
			v.label = _G[select(2, UnitPowerType("player"))]
			break
		end
	end
	
	local hasDualSpec = GetNumSpecGroups() > 1
	if hasDualSpec then
		self.db.dualProfile = self.db.dualProfile ~= false
	else
		-- always disable dual profile if the character doesn't have dual spec
		self.db.dualProfile = false
		dualProfile:Disable()
	end
	
	dualProfile:SetChecked(self.db.dualProfile)
	dualProfile:onClick()
	
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	
	self:LoadSettings()
end

function addon:CheckTalentSpec(activeSpec)
	activeSpec = activeSpec or GetActiveSpecGroup()
	local order, collapsed = "statCategoryOrder", "statCategoriesCollapsed"
	if activeSpec == 2 then
		order = order.."_2"
		collapsed = collapsed.."_2"
	end
	cvarOrder = order
	cvarCollapsed = collapsed
	CharacterStatsPane.orderCVarName = order
	CharacterStatsPane.collapsedCVarName = collapsed
	if self.db.dualProfile then
		db = self.db.profile[activeSpec]
	end
end

function addon:ACTIVE_TALENT_GROUP_CHANGED(activeSpec)
	self:CheckTalentSpec(activeSpec)
	if self.db.dualProfile then
		-- reset all the tables before loading the off spec data
		PAPERDOLL_STATCATEGORY_DEFAULTORDER = CopyTable(DEFAULT_CATEGORY_ORDER)
		PAPERDOLL_STATCATEGORIES = CopyTable(DEFAULT_CATEGORIES)
		self:LoadSettings()
		self:SelectCategory(nil)
	end
end

-- given the value, retrieves data about a stat from the stats table
function addon:GetStatInfo(value)
	for category, stats in pairs(statsDB) do
		for i, stat in ipairs(stats) do
			if stat.value == value then
				return stat.label, _G["STAT_CATEGORY_"..category]
			end
		end
	end
end

-- creates a new custom category
function addon:CreateCategory(name)
	if PAPERDOLL_STATCATEGORIES[name] or DEFAULT_CATEGORIES[name] then
		self:Print("A frame with this name already exists. Please choose a different name.")
		return false
	end
	db.categories[name] = {}
	self:InjectCategoryData(name, db.categories[name])
	self:InitStatCategories()
	return true
end

-- inserts a custom category into the default-tables
function addon:InjectCategoryData(name, stats)
	local id = #PAPERDOLL_STATCATEGORY_DEFAULTORDER + 1
	self:CheckStatWidget(id)
	db.registry[id] = name
	PAPERDOLL_STATCATEGORIES[name] = {id = id, stats = stats}
	PAPERDOLL_STATCATEGORY_DEFAULTORDER[id] = name
	_G["STAT_CATEGORY_"..name] = _G["STAT_CATEGORY_"..name] or name
	local categoryOrder = GetCVar(cvarOrder)
	if #categoryOrder > 0 and not (","..categoryOrder..","):match(","..id..",") then
		SetCVar(cvarOrder, format("%s,%s", GetCVar(cvarOrder), id))
	end
	SetCVarBitfield(cvarCollapsed, id, false)
end

-- resets default stat categories, including visibility and stats
function addon:ResetCategory(name)
	local defaultStats = CopyTable(DEFAULT_CATEGORIES[name].stats)
	db.categories[name] = defaultStats
	if self:IsHiddenCategory(name) then
		self:InjectCategoryData(name, defaultStats)
		self:InitStatCategories()
	else
		PAPERDOLL_STATCATEGORIES[name].stats = defaultStats
		PaperDollFrame_UpdateStats()
	end
end

function addon:DeleteCategory(name)
	if not self:IsDefaultCategory(name) then
		_G["STAT_CATEGORY_"..name] = nil
		-- only deselect if a custom category, since default ones can be restored
		self:SelectCategory(nil)
	end
	self:DeleteCategoryData(name)
	self:Print(format("Removed category '%s'.", _G["STAT_CATEGORY_"..name] or name))
	self:UpdateGUI()
	self:InitStatCategories()
end

local categoryOrder = {}

function addon:GetCategoryOrder()
	wipe(categoryOrder)
	local i = 0
	for id in gmatch(GetCVar(cvarOrder), "%d+") do
		i = i + 1
		categoryOrder[i] = tonumber(id)
	end
	return categoryOrder
end

function addon:DeleteCategoryData(name)
	local id = PAPERDOLL_STATCATEGORIES[name].id
	
	local categoryOrder = self:GetCategoryOrder()
	if #categoryOrder ~= 0 then
		for i = 1, #categoryOrder do
			-- remove the index whose value equals the ID of the removed frame
			if categoryOrder[i] == id then
				tremove(categoryOrder, i)
			end
			-- and lower all values greater than the desired ID by 1
			local v = categoryOrder[i]
			if v and v > id then
				categoryOrder[i] = v - 1
			end
		end
		SetCVar(cvarOrder, table.concat(categoryOrder, ","))
	end
	
	self:FixCategoryIndex(name)
	db.categories[name] = nil
	
	tremove(db.collapsed, id)
	for i = id, #db.collapsed + 1 do
		SetCVarBitfield(cvarCollapsed, i, db.collapsed[i] or false)
	end
end

function addon:FixCategoryIndex(name)
	local id = PAPERDOLL_STATCATEGORIES[name].id
	-- remove the category that uses the deleted ID, and lower all values that are greater than this ID by 1
	for i = 1, #PAPERDOLL_STATCATEGORY_DEFAULTORDER do
		if PAPERDOLL_STATCATEGORY_DEFAULTORDER[i] == name then
			tremove(PAPERDOLL_STATCATEGORY_DEFAULTORDER, i)
		end
		
		local category = PAPERDOLL_STATCATEGORIES[PAPERDOLL_STATCATEGORY_DEFAULTORDER[i]]
		if category and category.id > id then
			category.id = category.id - 1
		end
		if db.registry[i] == name then
			tremove(db.registry, i)
		end
	end
	PAPERDOLL_STATCATEGORIES[name] = nil
end

function addon:IsDefaultCategory(name)
	if DEFAULT_CATEGORIES[name] then
		return true
	end
	return false
end

function addon:IsHiddenCategory(name)
	if DEFAULT_CATEGORIES[name] and not PAPERDOLL_STATCATEGORIES[name] then
		return true
	end
	return false
end

function addon:IsModifiedCategory(name)
	local default, copy = DEFAULT_CATEGORIES[name], PAPERDOLL_STATCATEGORIES[name]
	if not (default and copy) then
		return false
	end
	default, copy = default.stats, copy.stats
	if not #default == #copy then
		return true
	end
	for i = 1, max(#default, #copy) do
		if default[i] ~= copy[i] then
			return true
		end
	end
	return false
end

function addon:InitStatCategories()
	PaperDoll_InitStatCategories(PAPERDOLL_STATCATEGORY_DEFAULTORDER, cvarOrder, cvarCollapsed, "player")
end

-- selects a category, setting appropriate values on all dropdowns
function addon:SelectCategory(name)
	if name then
		UIDropDownMenu_SetSelectedValue(frameList, name)
	else
		UIDropDownMenu_SetSelectedValue(frameList, nil)
		UIDropDownMenu_SetText(frameList, "Select category")
	end
	self:UpdateGUI()
end

function addon:UpdateGUI()
	local selectedCategory = self:GetSelectedCategory()
	local hidden = self:IsHiddenCategory(selectedCategory)
	
	deleteButton:Disable()
	resetButton:Disable()
	if selectedCategory then
		if hidden then
			UIDropDownMenu_SetText(frameList, GRAY_FONT_COLOR_CODE.._G["STAT_CATEGORY_"..selectedCategory])
		else
			UIDropDownMenu_SetText(frameList, _G["STAT_CATEGORY_"..selectedCategory])
			deleteButton:Enable()
		end
		if self:IsModifiedCategory(selectedCategory) or hidden then
			resetButton:Enable()
		end
	end
	
	info:SetShown(hidden)
	
	local stats = self:GetSelectedStats()
	if stats then
		for i, v in ipairs(stats) do
			self:SetStat(i, v)
		end
		-- add a frame at the bottom so we can add another stat
		self:SetStat(#stats + 1)
		self:HideUnused()
		self:UpdateArrows()
	else
		-- this is a default category that's been hidden, do not let the user edit it until it's shown again
		for i, v in ipairs(frameList.statLists) do
			v:Hide()
		end
	end
	
	self:UpdateSize()
end

function addon:HideUnused()
	for i = #self:GetSelectedStats() + 2, #frameList.statLists do
		self:GetStatFrame(i):Hide()
	end
end

-- set a stat to the given frame
function addon:SetStat(index, statValue)
	local frame = self:GetStatFrame(index)
	local stat, category = addon:GetStatInfo(statValue)
	if stat then
		UIDropDownMenu_SetSelectedValue(frame, statValue)
		UIDropDownMenu_SetText(frame, format("%s - %s", category, stat))
	else
		UIDropDownMenu_SetSelectedValue(frame, "none")
		UIDropDownMenu_SetText(frame, NONE)
	end
	frame:Show()
end

function addon:GetSelectedCategory()
	return UIDropDownMenu_GetSelectedValue(frameList)
end

-- gets an array of stat values based on the currently selected category
function addon:GetSelectedStats()
	local category = PAPERDOLL_STATCATEGORIES[self:GetSelectedCategory()]
	return category and category.stats
end

-- gets the dropdown frame for the given stat index, creates a new frame if necessary
function addon:GetStatFrame(index)
	return frameList.statLists[index] or self:NewStat(index)
end

-- creates a new stat group frame if we didn't have this many before
function addon:CheckStatWidget(id)
	if not _G["CharacterStatsPaneCategory"..id] then
		CreateFrame("Frame", "CharacterStatsPaneCategory"..id, CharacterStatsPaneScrollChild, "StatGroupTemplate")
	end
end

function addon:UpdateSize()
	local stats = self:GetSelectedStats()
	self:SetHeight(FRAME_HEIGHT_BASE + (DROPDOWN_OFFSET * ((stats and #stats or 0) + 1)))
end

-- sets swap-position-arrows visibility
function addon:UpdateArrows()
	local n = #self:GetSelectedStats()
	if n > 0 then
		for i = 1, n do
			local frame = self:GetStatFrame(i)
			frame.up:Show()
			frame.down:Show()
		end
		self:GetStatFrame(1).up:Hide()
		self:GetStatFrame(n).down:Hide()
	end
	local frame = self:GetStatFrame(n + 1)
	frame.up:Hide()
	frame.down:Hide()
end

function addon:Print(msg)
	print("|cff56a3ffPDFStats:|r", msg)
end